<?php

namespace Drupal\Tests\multiselect\FunctionalJavascript;

use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * Tests the multiselect element UI using JavaScript.
 *
 * @group multiselect
 */
class MultiselectElementsUITest extends WebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'multiselect',
    'field_ui',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Configuration for the multiselect field being tested.
   *
   * @var \Drupal\field\Entity\FieldConfig
   */
  protected $field;

  /**
   * Configuration for the entity form display that the multiselect field is on.
   *
   * @var \Drupal\Core\Entity\Entity\EntityFormDisplay
   */
  protected $formDisplay;

  /**
   * The test Content type.
   *
   * @var string
   */
  protected $type;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create a test user.
    $admin_user = $this->drupalCreateUser([
      'bypass node access',
      'administer content types',
      'administer node fields',
      'administer node form display',
      'administer node display',
    ]);
    $this->drupalLogin($admin_user);

    $this->createContentType(['type' => 'article']);
  }

  /**
   * Test for the jQuery.fn.selectAll functionality.
   */
  public function testSelectAll() {
    // Create the Text list field storage.
    FieldStorageConfig::create([
      'field_name' => 'field_text_list',
      'entity_type' => 'node',
      'bundle' => 'article',
      'type' => 'list_string',
      'settings' => [
        'allowed_values' => [
          'lorem' => 'Lorem',
          'ipsum' => 'Ipsum',
          'dolor' => 'Dolor',
        ],
      ],
      'cardinality' => -1,
    ])->save();

    // Add the Text list field to Article content type.
    $this->field = FieldConfig::create([
      'field_name' => 'field_text_list',
      'entity_type' => 'node',
      'bundle' => 'article',
      'enabled' => TRUE,
    ]);
    $this->field->save();

    // Enable Multiselect widget.
    $this->formDisplay = EntityFormDisplay::load('node.article.default');
    $this->formDisplay->setComponent($this->field->getName(), [
      'type' => 'multiselect',
    ])->save();

    // Create Article.
    $this->drupalCreateNode([
      'type' => 'article',
      'title' => 'Test article',
    ]);

    $this->drupalGet('/node/1/edit');
    $page = $this->getSession()->getPage();
    $add_btn = $page->find('css', '.multiselect-add');

    // The field field_text_list doesn't have a default value and
    // we expected that the Selected Options list is empty.
    $this->assertSession()->elementsCount('css', '#edit-field-text-list option', 0);

    // Move options 'lorem' and 'dolor' to the Selected Options list.
    $lorem = $page->find('css', '#edit-field-text-list-available option[value="lorem"]');
    $lorem->click();
    $add_btn->click();

    $dolor = $page->find('css', '#edit-field-text-list-available option[value="dolor"]');
    $dolor->click();
    $add_btn->click();

    // Save the test node.
    $page->pressButton('Save');

    // Visit the edit page to check the success of saving changes.
    $this->drupalGet('/node/1/edit');

    // We moved 'lorem' and 'dolor' to the Selected Options list and
    // expected them in it.
    $this->assertSession()->elementsCount('css', '#edit-field-text-list option', 2);
  }

}
